@@ -15,14 +15,13 @@ except ImportError:
     class ImproperlyConfigured(Exception):
         pass
 
-from six.moves import urllib_parse as urlparse
+from six.moves import urllib
 from six import string_types
 
-
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
-VERSION = '0.4.0'
+VERSION = '0.4.3'
 __author__ = 'joke2k'
 __version__ = tuple(VERSION.split('.'))
 
@@ -31,6 +30,20 @@ __version__ = tuple(VERSION.split('.'))
 def _cast_int(v):
     return int(v) if hasattr(v, 'isdigit') and v.isdigit() else v
 
+def _cast_urlstr(v):
+    return urllib.parse.unquote_plus(v) if isinstance(v, str) else v
+
+# back compatibility with redis_cache package
+DJANGO_REDIS_DRIVER = 'django_redis.cache.RedisCache'
+DJANGO_REDIS_CACHE_DRIVER = 'redis_cache.RedisCache'
+
+REDIS_DRIVER = DJANGO_REDIS_DRIVER
+try:
+    import redis_cache
+    REDIS_DRIVER = DJANGO_REDIS_CACHE_DRIVER
+except:
+    pass
+
 
 class NoValue(object):
 
@@ -50,9 +63,10 @@ class Env(object):
             ...
     """
 
+    ENVIRON = os.environ
     NOTSET = NoValue()
     BOOLEAN_TRUE_STRINGS = ('true', 'on', 'ok', 'y', 'yes', '1')
-    URL_CLASS = urlparse.ParseResult
+    URL_CLASS = urllib.parse.ParseResult
     DEFAULT_DATABASE_ENV = 'DATABASE_URL'
     DB_SCHEMES = {
         'postgres': 'django.db.backends.postgresql_psycopg2',
@@ -63,6 +77,8 @@ class Env(object):
         'mysql': 'django.db.backends.mysql',
         'mysql2': 'django.db.backends.mysql',
         'mysqlgis': 'django.contrib.gis.db.backends.mysql',
+        'oracle': 'django.db.backends.oracle',
+        'redshift': 'django_redshift_backend',
         'spatialite': 'django.contrib.gis.db.backends.spatialite',
         'sqlite': 'django.db.backends.sqlite3',
         'ldap': 'ldapdb.backends.ldap',
@@ -77,10 +93,10 @@ class Env(object):
         'locmemcache': 'django.core.cache.backends.locmem.LocMemCache',
         'memcache': 'django.core.cache.backends.memcached.MemcachedCache',
         'pymemcache': 'django.core.cache.backends.memcached.PyLibMCCache',
-        'rediscache': 'django_redis.cache.RedisCache',
-        'redis': 'django_redis.cache.RedisCache',
+        'rediscache': REDIS_DRIVER,
+        'redis': REDIS_DRIVER,
     }
-    _CACHE_BASE_OPTIONS = ['TIMEOUT', 'KEY_PREFIX', 'VERSION', 'KEY_FUNCTION']
+    _CACHE_BASE_OPTIONS = ['TIMEOUT', 'KEY_PREFIX', 'VERSION', 'KEY_FUNCTION', 'BINARY']
 
     DEFAULT_EMAIL_ENV = 'EMAIL_URL'
     EMAIL_SCHEMES = {
@@ -170,7 +186,7 @@ class Env(object):
         """
         :rtype: urlparse.ParseResult
         """
-        return self.get_value(var, cast=urlparse.urlparse, default=default, parse_default=True)
+        return self.get_value(var, cast=urllib.parse.urlparse, default=default, parse_default=True)
 
     def db_url(self, var=DEFAULT_DATABASE_ENV, default=NOTSET, engine=None):
         """Returns a config dictionary, defaulting to DATABASE_URL.
@@ -246,7 +262,7 @@ class Env(object):
                     cast = var_info
 
         try:
-            value = os.environ[var]
+            value = self.ENVIRON[var]
         except KeyError:
             if default is self.NOTSET:
                 error_msg = "Set the {0} environment variable".format(var)
@@ -322,7 +338,7 @@ class Env(object):
     @classmethod
     def db_url_config(cls, url, engine=None):
         """Pulled from DJ-Database-URL, parse an arbitrary Database URL.
-        Support currently exists for PostgreSQL, PostGIS, MySQL and SQLite.
+        Support currently exists for PostgreSQL, PostGIS, MySQL, Oracle and SQLite.
 
         SQLite connects to file based databases. The same URL format is used, omitting the hostname,
         and using the "file" portion as the filename of the database.
@@ -330,7 +346,7 @@ class Env(object):
 
         >>> from environ import Env
         >>> Env.db_url_config('sqlite:////full/path/to/your/file.sqlite')
-        {'ENGINE': 'django.db.backends.sqlite3', 'HOST': None, 'NAME': '/full/path/to/your/file.sqlite', 'PASSWORD': None, 'PORT': None, 'USER': None}
+        {'ENGINE': 'django.db.backends.sqlite3', 'HOST': '', 'NAME': '/full/path/to/your/file.sqlite', 'PASSWORD': '', 'PORT': '', 'USER': ''}
         >>> Env.db_url_config('postgres://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn')
         {'ENGINE': 'django.db.backends.postgresql_psycopg2', 'HOST': 'ec2-107-21-253-135.compute-1.amazonaws.com', 'NAME': 'd8r82722r2kuvn', 'PASSWORD': 'wegauwhgeuioweg', 'PORT': 5431, 'USER': 'uf07k1i6d8ia0v'}
 
@@ -345,13 +361,13 @@ class Env(object):
                     'NAME': ':memory:'
                 }
                 # note: no other settings are required for sqlite
-            url = urlparse.urlparse(url)
+            url = urllib.parse.urlparse(url)
 
         config = {}
 
         # Remove query strings.
         path = url.path[1:]
-        path = path.split('?', 2)[0]
+        path = urllib.parse.unquote_plus(path.split('?', 2)[0])
 
         # if we are using sqlite and we have no path, then assume we
         # want an in-memory database (this is the behaviour of sqlalchemy)
@@ -364,16 +380,27 @@ class Env(object):
 
         # Update with environment configuration.
         config.update({
-            'NAME': path,
-            'USER': url.username,
-            'PASSWORD': url.password,
-            'HOST': url.hostname,
-            'PORT': _cast_int(url.port),
+            'NAME': path or '',
+            'USER': _cast_urlstr(url.username) or '',
+            'PASSWORD': _cast_urlstr(url.password) or '',
+            'HOST': url.hostname or '',
+            'PORT': _cast_int(url.port) or '',
         })
 
+        if url.scheme == 'oracle' and path == '':
+            config['NAME'] = config['HOST']
+            config['HOST'] = ''
+
+        if url.scheme == 'oracle':
+            # Django oracle/base.py strips port and fails on non-string value
+            if not config['PORT']:
+                del(config['PORT'])
+            else:
+                config['PORT'] = str(config['PORT'])
+
         if url.query:
             config_options = {}
-            for k, v in urlparse.parse_qs(url.query).items():
+            for k, v in urllib.parse.parse_qs(url.query).items():
                 if k.upper() in cls._DB_BASE_OPTIONS:
                     config.update({k.upper(): _cast_int(v[0])})
                 else:
@@ -399,7 +426,7 @@ class Env(object):
         :param backend:
         :return:
         """
-        url = urlparse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
+        url = urllib.parse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
 
         location = url.netloc.split(',')
         if len(location) == 1:
@@ -410,19 +437,27 @@ class Env(object):
             'LOCATION': location,
         }
 
+        # Add the drive to LOCATION
         if url.scheme == 'filecache':
             config.update({
                 'LOCATION': url.netloc + url.path,
             })
 
-        if url.path and url.scheme in ['memcache', 'pymemcache', 'rediscache']:
+        if url.path and url.scheme in ['memcache', 'pymemcache']:
             config.update({
                 'LOCATION': 'unix:' + url.path,
             })
+        elif url.scheme.startswith('redis'):
+            if url.hostname:
+                scheme = url.scheme.replace('cache', '')
+            else:
+                scheme = 'unix'
+
+            config['LOCATION'] = scheme + '://' + url.netloc + url.path
 
         if url.query:
             config_options = {}
-            for k, v in urlparse.parse_qs(url.query).items():
+            for k, v in urllib.parse.parse_qs(url.query).items():
                 opt = {k.upper(): _cast_int(v[0])}
                 if k.upper() in cls._CACHE_BASE_OPTIONS:
                     config.update(opt)
@@ -441,17 +476,17 @@ class Env(object):
 
         config = {}
 
-        url = urlparse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
+        url = urllib.parse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
 
         # Remove query strings
         path = url.path[1:]
-        path = path.split('?', 2)[0]
+        path = urllib.parse.unquote_plus(path.split('?', 2)[0])
 
         # Update with environment configuration
         config.update({
             'EMAIL_FILE_PATH': path,
-            'EMAIL_HOST_USER': url.username,
-            'EMAIL_HOST_PASSWORD': url.password,
+            'EMAIL_HOST_USER': _cast_urlstr(url.username),
+            'EMAIL_HOST_PASSWORD': _cast_urlstr(url.password),
             'EMAIL_HOST': url.hostname,
             'EMAIL_PORT': _cast_int(url.port),
         })
@@ -470,7 +505,7 @@ class Env(object):
 
         if url.query:
             config_options = {}
-            for k, v in urlparse.parse_qs(url.query).items():
+            for k, v in urllib.parse.parse_qs(url.query).items():
                 opt = {k.upper(): _cast_int(v[0])}
                 if k.upper() in cls._EMAIL_BASE_OPTIONS:
                     config.update(opt)
@@ -484,11 +519,11 @@ class Env(object):
     def search_url_config(cls, url, engine=None):
         config = {}
 
-        url = urlparse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
+        url = urllib.parse.urlparse(url) if not isinstance(url, cls.URL_CLASS) else url
 
         # Remove query strings.
         path = url.path[1:]
-        path = path.split('?', 2)[0]
+        path = urllib.parse.unquote_plus(path.split('?', 2)[0])
 
         if url.scheme not in cls.SEARCH_SCHEMES:
             raise ImproperlyConfigured('Invalid search schema %s' % url.scheme)
@@ -497,7 +532,7 @@ class Env(object):
         # check commons params
         params = {}
         if url.query:
-            params = urlparse.parse_qs(url.query)
+            params = urllib.parse.parse_qs(url.query)
             if 'EXCLUDED_INDEXES' in params.keys():
                 config['EXCLUDED_INDEXES'] = params['EXCLUDED_INDEXES'][0].split(',')
             if 'INCLUDE_SPELLING' in params.keys():
@@ -516,7 +551,7 @@ class Env(object):
             path = path[:-1]
 
         if url.scheme == 'solr':
-            config['URL'] = urlparse.urlunparse(('http',) + url[1:2] + (path,) + ('', '', ''))
+            config['URL'] = urllib.parse.urlunparse(('http',) + url[1:2] + (path,) + ('', '', ''))
             if 'TIMEOUT' in params.keys():
                 config['TIMEOUT'] = cls.parse_value(params['TIMEOUT'][0], int)
             return config
@@ -532,7 +567,7 @@ class Env(object):
                 path = ""
                 index = split[0]
 
-            config['URL'] = urlparse.urlunparse(('http',) + url[1:2] + (path,) + ('', '', ''))
+            config['URL'] = urllib.parse.urlunparse(('http',) + url[1:2] + (path,) + ('', '', ''))
             if 'TIMEOUT' in params.keys():
                 config['TIMEOUT'] = cls.parse_value(params['TIMEOUT'][0], int)
             config['INDEX_NAME'] = index
@@ -554,8 +589,8 @@ class Env(object):
 
         return config
 
-    @staticmethod
-    def read_env(env_file=None, **overrides):
+    @classmethod
+    def read_env(cls, env_file=None, **overrides):
         """Read a .env file into os.environ.
 
         If not given a path to a dotenv path, does filthy magic stack backtracking
@@ -569,14 +604,18 @@ class Env(object):
             frame = sys._getframe()
             env_file = os.path.join(os.path.dirname(frame.f_back.f_code.co_filename), '.env')
             if not os.path.exists(env_file):
-                warnings.warn("not reading %s - it doesn't exist." % env_file)
+                warnings.warn(
+                    "%s doesn't exist - if you're not configuring your "
+                    "environment separately, create one." % env_file)
                 return
 
         try:
             with open(env_file) if isinstance(env_file, string_types) else env_file as f:
                 content = f.read()
         except IOError:
-            warnings.warn("not reading %s - it doesn't exist." % env_file)
+            warnings.warn(
+                "Error reading %s - if you're not configuring your "
+                "environment separately, check this." % env_file)
             return
 
         logger.debug('Read environment variables from: {0}'.format(env_file))
@@ -591,11 +630,11 @@ class Env(object):
                 m3 = re.match(r'\A"(.*)"\Z', val)
                 if m3:
                     val = re.sub(r'\\(.)', r'\1', m3.group(1))
-                os.environ.setdefault(key, str(val))
+                cls.ENVIRON.setdefault(key, str(val))
 
         # set defaults
         for key, value in overrides.items():
-            os.environ.setdefault(key, value)
+            cls.ENVIRON.setdefault(key, value)
 
 
 class Path(object):
@@ -720,9 +759,9 @@ class Path(object):
 
 
 def register_scheme(scheme):
-    for method in dir(urlparse):
+    for method in dir(urllib.parse):
         if method.startswith('uses_'):
-            getattr(urlparse, method).append(scheme)
+            getattr(urllib.parse, method).append(scheme)
 
 
 def register_schemes(schemes):
